"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const framework_1 = require("./homebridge/framework");
const platform_1 = require("./platform");
const platform_config_1 = require("./platform_config");
const auth_1 = require("./tion/auth");
const api_1 = require("./tion/api");
const factory_1 = require("./tion/devices/factory");
const accessories_factory_1 = require("./accessories_factory");
let GenerateUuid;
let Service;
let Characteristic;
let Accessory;
let User;
class TionPlatformWrapper extends framework_1.HomebridgePlatform {
    constructor(log, config, hbApi) {
        super(log, config, hbApi);
        this.configureAccessory = (accessory) => {
            return this.instance && this.instance.loadCachedAccessory(accessory);
        };
        if (!log) {
            throw new Error('Tion: log service not found. Probably incompatible Homebridge version');
        }
        if (!config || typeof config !== 'object') {
            log.error('config not set, stopping platform');
            return;
        }
        if (!hbApi) {
            log.error('api service not found, probably incompatible Homebridge version, stopping platform');
            return;
        }
        if (!(0, platform_config_1.validate)(log, (0, platform_config_1.sanitize)(log, config))) {
            this.log.error('config invalid, stopping platform');
            return;
        }
        const authStorage = new auth_1.TionFilesystemAuthStorage(log, User.persistPath());
        const authApi = new auth_1.TionAuthApi(log, config, authStorage);
        const api = new api_1.TionApi(log, config, authApi);
        const devicesFactory = new factory_1.TionDevicesFactory(log, config, api, Service, Characteristic);
        const accessoriesFactory = new accessories_factory_1.AccessoriesFactory(log, Service, Characteristic, Accessory, GenerateUuid);
        this.instance = new platform_1.TionPlatform(log, hbApi, api, devicesFactory, accessoriesFactory);
    }
}
exports.default = (homebridge) => {
    GenerateUuid = homebridge.hap.uuid.generate;
    Service = homebridge.hap.Service;
    Characteristic = homebridge.hap.Characteristic;
    Accessory = homebridge.platformAccessory;
    User = homebridge.user;
    homebridge.registerPlatform(platform_config_1.PluginName, platform_config_1.PlatformName, TionPlatformWrapper, true);
};
//# sourceMappingURL=index.js.map