"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TionFilesystemAuthStorage = exports.TionAuthApi = exports.AuthError = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const url_1 = __importDefault(require("url"));
const node_fetch_1 = __importDefault(require("node-fetch"));
const framework_1 = require("../homebridge/framework");
class AuthError extends Error {
    constructor(message, statusCode) {
        const trueProto = new.target.prototype;
        super(message);
        Object.setPrototypeOf(this, trueProto);
        this.statusCode = statusCode;
    }
}
exports.AuthError = AuthError;
class TionAuthApi {
    constructor(log, config, storage) {
        this.log = log;
        this.config = config;
        this.authStorage = storage;
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                this.myAuthData = yield this.authStorage.load();
            }
            catch (err) {
                err = null; // relax lint
            }
        });
    }
    getAccessToken() {
        return this.myAuthData && this.myAuthData.access_token;
    }
    authenticateUsingPassword() {
        return __awaiter(this, void 0, void 0, function* () {
            this.log.debug('TionAuthApi - authenticating using password');
            yield this._internalAuthenticate({
                grant_type: 'password',
                username: this.config.userName,
                password: this.config.password,
                client_id: TionAuthApi.clientId,
                client_secret: TionAuthApi.clientSecret,
            });
            yield this.authStorage.save(this.myAuthData);
            return this.myAuthData && this.myAuthData.access_token;
        });
    }
    authenticateUsingRefreshToken() {
        return __awaiter(this, void 0, void 0, function* () {
            this.log.debug('TionAuthApi - authenticating using refresh token');
            yield this._internalAuthenticate({
                grant_type: 'refresh_token',
                refresh_token: this.myAuthData.refresh_token,
                client_id: TionAuthApi.clientId,
                client_secret: TionAuthApi.clientSecret,
            });
            yield this.authStorage.save(this.myAuthData);
            return this.myAuthData.access_token;
        });
    }
    _internalAuthenticate(params) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const authResult = yield (0, node_fetch_1.default)(TionAuthApi.oAuthUrl, {
                    method: 'POST',
                    body: new url_1.default.URLSearchParams(params),
                });
                if (authResult.ok) {
                    this.myAuthData = yield authResult.json();
                }
                else {
                    throw new AuthError(authResult.statusText, authResult.status);
                }
            }
            catch (err) {
                this.log.error('TionAuthApi._internalAuthenticate: ', err.message);
                this.log.debug('TionAuthApi._internalAuthenticate: ', err);
                throw err;
            }
        });
    }
}
exports.TionAuthApi = TionAuthApi;
TionAuthApi.oAuthUrl = 'https://api2.magicair.tion.ru/idsrv/oauth2/token';
TionAuthApi.clientId = 'a750d720-e146-47b0-b414-35e3b1dd7862';
TionAuthApi.clientSecret = 'DTT2jJnY3k2H2GyZ';
TionAuthApi.refreshClientId = '8b96527d-7632-4d56-bf75-3d1097e99d0e';
TionAuthApi.refreshClientSecret = 'qwerty';
class TionFilesystemAuthStorage {
    constructor(log, basePath) {
        this.log = log;
        this.basePath = basePath;
        if (!fs_1.default.existsSync(basePath)) {
            fs_1.default.mkdirSync(basePath, { recursive: true });
        }
    }
    save(authData) {
        return __awaiter(this, void 0, void 0, function* () {
            fs_1.default.writeFileSync(this.getStatePath(), JSON.stringify(authData, null, 4), { encoding: 'utf8' });
            this.log.debug('Auth tokens persisted');
        });
    }
    load() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const text = fs_1.default.readFileSync(this.getStatePath(), { encoding: 'utf8' });
                const ret = JSON.parse(text);
                this.log.debug('Got persisted auth tokens');
                return ret;
            }
            catch (err) {
                this.log.debug('Auth tokens not persisted');
                throw err;
            }
        });
    }
    getStatePath() {
        return path_1.default.join(this.basePath, 'homebridge-tion.auth.json');
    }
}
exports.TionFilesystemAuthStorage = TionFilesystemAuthStorage;
//# sourceMappingURL=auth.js.map