"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TionDeviceBase = void 0;
const framework_1 = require("../../homebridge/framework");
const platform_config_1 = require("../../platform_config");
const api_1 = require("../api");
const state_1 = require("../state");
const command_1 = require("../command");
class TionDeviceBase {
    constructor(device, zone, log, config, api, serviceRegistry, characteristicRegistry) {
        this.zoneId = zone.guid;
        this.id = device.guid;
        this.name = device.name;
        this.modelName = device.type;
        this.mac = device.mac;
        this.firmwareRevision = device.firmware;
        this.hardwareRevision = device.hardware;
        this.isOnline = true;
        this.isCommandRunning = false;
        this.log = log;
        this.config = config;
        this.api = api;
        this.serviceRegistry = serviceRegistry;
        this.characteristicRegistry = characteristicRegistry;
        this.accessories = [];
    }
    findDeviceInState(state) {
        let device = null;
        let zone = null;
        state.zones.forEach(z => {
            if (device) {
                return;
            }
            z.devices.forEach(d => {
                if (device) {
                    return;
                }
                if (d.guid === this.id) {
                    device = d;
                    zone = z;
                }
            });
        });
        if (!device) {
            this.log.error(`Device ${this.name} (${this.id}) not found in remote state`);
        }
        return {
            device,
            zone,
        };
    }
    setState(command) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                this.isCommandRunning = true;
                yield this.api.execDeviceCommand(this.id, command);
            }
            finally {
                this.isCommandRunning = false;
            }
        });
    }
    getState(callback, getter) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const state = yield this.api.getSystemState();
                if (this.parseState(state)) {
                    if (!this.isOnline) {
                        this.accessories.forEach(a => (a.reachable = false));
                        this.log.error(`Device ${this.name} (${this.id}) not reachable`);
                        return callback('Not reachable');
                    }
                    callback(null, getter());
                }
                else {
                    this.log.error(`Device ${this.name} (${this.id}) cannot parse state`);
                    callback('Cannot parse state');
                }
            }
            catch (err) {
                this.log.error('Cannot get system state');
                this.log.error(err);
                callback('Cannot get state');
            }
        });
    }
    rollbackCharacteristic(service, characteristic, value) {
        setTimeout(() => {
            service.updateCharacteristic(characteristic, value);
        }, 100);
    }
}
exports.TionDeviceBase = TionDeviceBase;
//# sourceMappingURL=base.js.map