"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TionDevicesFactory = void 0;
const framework_1 = require("../../homebridge/framework");
const platform_config_1 = require("../../platform_config");
const state_1 = require("../state");
const api_1 = require("../api");
const station_1 = require("./station");
const breezer_1 = require("./breezer");
const co2plus_1 = require("./co2plus");
const supported_device_types_1 = require("./supported_device_types");
class TionDevicesFactory {
    constructor(log, config, api, serviceRegistry, characteristicRegistry) {
        this.log = log;
        this.config = config;
        this.api = api;
        this.serviceRegistry = serviceRegistry;
        this.characteristicRegistry = characteristicRegistry;
    }
    createDevices(existingDevices, location) {
        const devices = location.zones.reduce((all, zone) => {
            const zoneDevices = [];
            zone.devices.forEach(d => {
                if (!existingDevices.find(ex => ex.id === d.guid)) {
                    const device = this.createDevice(d, zone);
                    if (device) {
                        zoneDevices.push(device);
                    }
                }
            });
            return all.concat(zoneDevices);
        }, []);
        return existingDevices.concat(devices);
    }
    createDevice(device, zone) {
        switch (device.type) {
            default:
                this.log.warn(`Unsupported device type ${device.type}. Please contact plugin developer to add device support.`);
                return null;
            case supported_device_types_1.SupportedDeviceTypes.MagicAirStation:
                return new station_1.TionMagicAirStation(device, zone, this.log, this.config, this.api, this.serviceRegistry, this.characteristicRegistry);
            case supported_device_types_1.SupportedDeviceTypes.Breezer4S:
            case supported_device_types_1.SupportedDeviceTypes.Breezer3S:
            case supported_device_types_1.SupportedDeviceTypes.BreezerO2:
                return new breezer_1.TionBreezer(device, zone, this.log, this.config, this.api, this.serviceRegistry, this.characteristicRegistry);
            case supported_device_types_1.SupportedDeviceTypes.CO2Plus:
                return new co2plus_1.TionCO2Plus(device, zone, this.log, this.config, this.api, this.serviceRegistry, this.characteristicRegistry);
        }
    }
}
exports.TionDevicesFactory = TionDevicesFactory;
//# sourceMappingURL=factory.js.map