"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TionMagicAirStation = void 0;
const framework_1 = require("../../homebridge/framework");
const base_1 = require("./base");
const state_1 = require("../state");
class TionMagicAirStation extends base_1.TionDeviceBase {
    constructor() {
        super(...arguments);
        this.co2Level = 0;
        this.temperature = 0;
        this.humidity = 0;
    }
    addEventHandlers(accessory) {
        accessory.reachable = true;
        if (this.accessories.find(a => a.UUID === accessory.UUID)) {
            return;
        }
        this.accessories.push(accessory);
        const co2Sensor = accessory.getService(this.serviceRegistry.CarbonDioxideSensor);
        if (co2Sensor) {
            co2Sensor
                .getCharacteristic(this.characteristicRegistry.CarbonDioxideDetected)
                .on('get', callback => this.getState(callback, () => this.carbonDioxideDetected()));
            co2Sensor
                .getCharacteristic(this.characteristicRegistry.CarbonDioxideLevel)
                .on('get', callback => this.getState(callback, () => this.co2Level));
        }
        const temperatureSensor = accessory.getService(this.serviceRegistry.TemperatureSensor);
        if (temperatureSensor) {
            temperatureSensor
                .getCharacteristic(this.characteristicRegistry.CurrentTemperature)
                .on('get', callback => this.getState(callback, () => this.temperature));
        }
        const humiditySensor = accessory.getService(this.serviceRegistry.HumiditySensor);
        if (humiditySensor) {
            humiditySensor
                .getCharacteristic(this.characteristicRegistry.CurrentRelativeHumidity)
                .on('get', callback => this.getState(callback, () => this.humidity));
        }
    }
    updateState(state) {
        this.parseState(state);
        this.accessories.forEach(accessory => {
            accessory.reachable = this.isOnline;
            const co2Sensor = accessory.getService(this.serviceRegistry.CarbonDioxideSensor);
            if (co2Sensor) {
                co2Sensor.setCharacteristic(this.characteristicRegistry.CarbonDioxideDetected, this.carbonDioxideDetected());
                co2Sensor.setCharacteristic(this.characteristicRegistry.CarbonDioxideLevel, this.co2Level);
            }
            const temperatureSensor = accessory.getService(this.serviceRegistry.TemperatureSensor);
            if (temperatureSensor) {
                temperatureSensor.setCharacteristic(this.characteristicRegistry.CurrentTemperature, this.temperature);
            }
            const humiditySensor = accessory.getService(this.serviceRegistry.HumiditySensor);
            if (humiditySensor) {
                humiditySensor.setCharacteristic(this.characteristicRegistry.CurrentRelativeHumidity, this.humidity);
            }
        });
    }
    parseState(state) {
        var _a, _b, _c;
        const { device } = this.findDeviceInState(state);
        if (!device) {
            return false;
        }
        this.isOnline = device.is_online;
        this.co2Level = ((_a = device.data) === null || _a === void 0 ? void 0 : _a.co2) || 0;
        this.temperature = ((_b = device.data) === null || _b === void 0 ? void 0 : _b.temperature) || 0;
        this.humidity = ((_c = device.data) === null || _c === void 0 ? void 0 : _c.humidity) || 0;
        return true;
    }
    carbonDioxideDetected() {
        return this.co2Level > this.config.co2Threshold ? 1 : 0;
    }
}
exports.TionMagicAirStation = TionMagicAirStation;
//# sourceMappingURL=station.js.map